@@ -1,3 +1,14 @@
+import gi
+gi.require_version('Gtk', '3.0')
+from gi.repository import Gtk, Gio, GLib, GdkPixbuf
+
+try:
+    gi.require_version('AppIndicator3', '0.1')
+    from gi.repository import AppIndicator3
+except:
+    gi.require_version('AyatanaAppIndicator3', '0.1')
+    from gi.repository import AyatanaAppIndicator3 as AppIndicator3
+
 import sys
 import os
 import time
@@ -7,19 +18,10 @@ import syslog
 import appdirs
 import configparser
 
-import gi
-gi.require_version('Gtk', '3.0')
-gi.require_version('AppIndicator3', '0.1')
-from gi.repository import Gtk, Gio, GLib, GdkPixbuf, AppIndicator3
-
-try:
-    from bluedo.bt_rssi import BluetoothRSSI
-except ImportError:
-    from bt_rssi import BluetoothRSSI
 
 class BlueDo(Gtk.Application):
     project_name = 'bluedo'
-    project_version = .56
+    project_version = 2.1
     config_path = appdirs.user_config_dir(project_name) + '/' + project_name + '.ini'
     config_section = 'CONFIG'
     run_path = os.path.dirname(os.path.realpath(__file__)) + '/'
@@ -39,11 +41,10 @@ class BlueDo(Gtk.Application):
     config = None
     minimized = False # Start up minimized
     nearby_devices = []
-    #start_pingthread = None
     ping_stop = False # Signal background ping thread to shut down
     scan_stop = False # Signal background device thread to shut down
     window_width = 1020
-    window_heigth = 700
+    window_heigth = 750
 
     def __init__(self, *args, **kwargs):
         super().__init__(
@@ -118,7 +119,7 @@ class BlueDo(Gtk.Application):
         if len(self.bt_address) > 0:
             self.button_enabled.set_active(self.enabled)
             self.menuitem_enable.set_active(self.enabled)
-        else: 
+        else:
             self.button_enabled.set_sensitive(False)
             self.menuitem_enable.set_sensitive(False)
 
@@ -180,7 +181,6 @@ class BlueDo(Gtk.Application):
             self.window.hide()
 
         self.start_devicethread() # Look for paired bluetooth devices
-        #self.start_pingthread = self.start_pingthread() # Ping selected device for RSSI
         self.start_pingthread() # Ping selected device for RSSI
 
         Gtk.main()
@@ -207,9 +207,13 @@ class BlueDo(Gtk.Application):
         ''' Switch demo animation to static image '''
 
         path_picture = self.run_path + "unlocked.png"
-        pixbuf = GdkPixbuf.PixbufAnimation.new_from_file(path_picture)
-        self.demo_image.set_from_animation(pixbuf)
+        if self.debug:
+            syslog.syslog("Showing picture <%s>" % path_picture)
+
+        self.demo_image.set_from_animation(GdkPixbuf.PixbufAnimation.new_from_file(path_picture))
+        self.instructions_viewport.remove(self.instructions_viewport.get_child())
         self.instructions_viewport.add(self.demo_image)
+        self.demo_image.show()
 
 
 #
@@ -344,13 +348,13 @@ class BlueDo(Gtk.Application):
 
     def autostart_clicked(self, state):
         ''' Toggle autostart '''
-        
+
         self.create_autostart(ensure=self.menuitem_autostart.get_active())
 
     def bluetoothsettings_clicked(self, state):
         ''' Open gnome-control-center on bluetooth '''
 
-        os.system('gnome-control-center bluetooth')
+        os.system('gnome-control-center bluetooth &')
 
 
 #
@@ -384,7 +388,7 @@ class BlueDo(Gtk.Application):
         self.config.set(self.config_section, 'minimized', str(self.minimized))
         self.config.set(self.config_section, 'enabled', str(self.enabled))
 
-        with open(self.config_path, 'w') as f:
+        with open(self.config_path, mode='w', encoding="utf8") as f:
             self.config.write(f)
 
     def load_config(self):
@@ -411,7 +415,7 @@ class BlueDo(Gtk.Application):
         self.bt_name = self.config.get(self.config_section, 'bt_name', fallback='(current)')
         if "true" in self.config.get(self.config_section, 'debug', fallback='false').lower():
             self.debug = True
-        if "true" in self.config.get(self.config_section, 'here_unlock', fallback='false').lower():
+        if "true" in self.config.get(self.config_section, 'here_unlock', fallback='true').lower():
             self.check_hereunlock.set_active(True)
         if "true" in self.config.get(self.config_section, 'check_resume', fallback='false').lower():
             self.check_resume.set_active(True)
@@ -419,7 +423,7 @@ class BlueDo(Gtk.Application):
             self.check_unmute.set_active(True)
         if "true" in self.config.get(self.config_section, 'here_run', fallback='false').lower():
             self.check_hererun.set_active(True)
-        if "true" in self.config.get(self.config_section, 'away_lock', fallback='false').lower():
+        if "true" in self.config.get(self.config_section, 'away_lock', fallback='true').lower():
             self.check_awaylock.set_active(True)
         if "true" in self.config.get(self.config_section, 'away_mute', fallback='false').lower():
             self.check_awaymute.set_active(True)
@@ -438,7 +442,7 @@ class BlueDo(Gtk.Application):
         """ Create autostart file. Or deletes if not ensure. """
 
         if ensure:
-            with open(self.autostart_dir + self.project_name + '.desktop', 'w') as f:
+            with open(self.autostart_dir + self.project_name + '.desktop', mode='w', encoding="utf8") as f:
                 f.write("""[Desktop Entry]
 Name=BlueDo
 Comment=Bluetooth proximity automation
@@ -471,36 +475,36 @@ Categories=Utility;
                 ('[TV] tv10b', '78:BD:BC:6E:C5:A3'),
             ]
         else:
-            proc = subprocess.Popen(
-                args=cmd, 
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE,
-                universal_newlines=True
-            )
+            with subprocess.Popen(
+                    args=cmd,
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
+                    universal_newlines=True
+            ) as proc:
 
-            for line in iter(proc.stdout.readline, b''):
-                if BlueDo.debug:
-                    print("bluetoothctl line: " + line)
-                if line.strip() == '': # iter calls until output is ''
-                    break
-                if line == 'No default controller available\n':
-                    break
-                addr = ''
-                try:
-                    addr = line.split()[1]
-                except IndexError:
-                    print("Info: Skipping empty line from bluetoothctl.")
-                    continue
-                name = ' '.join(line.split()[2:])
-                devices += [(name, addr)]
+                for line in iter(proc.stdout.readline, b''):
+                    if BlueDo.debug:
+                        syslog.syslog("bluetoothctl line: " + line)
+                    if line.strip() == '': # iter calls until output is ''
+                        break
+                    if line == 'No default controller available\n':
+                        break
+                    addr = ''
+                    try:
+                        addr = line.split()[1]
+                    except IndexError:
+                        syslog.syslog("Info: Skipping empty line from bluetoothctl.")
+                        continue
+                    name = ' '.join(line.split()[2:])
+                    devices += [(name, addr)]
 
-            proc.communicate() # Allow cmd to exit cleanly
+                proc.communicate() # Allow cmd to exit cleanly
         return devices
 
     def start_devicethread(self):
         if self.debug:
             syslog.syslog("Staring thread update_combodevices")
-        t = threading.Thread(target = self.update_combodevices) 
+        t = threading.Thread(target = self.update_combodevices)
         t.start()
 
     def update_combodevices(self):
@@ -511,6 +515,11 @@ Categories=Utility;
             if self.scan_stop: # Check if asked to shut down
                 break
 
+            # Don't update combo if open
+            if self.combo_device.get_property("popup-shown"):
+                time.sleep(self.interval)
+                continue
+
             # Save contents of combo_device, clear and reset
             newscan = self.bluetooth_list()
 
@@ -556,7 +565,7 @@ Categories=Utility;
                 'away_callback': self.away_callback
             }
         )
-        
+
         thread.daemon = True # Daemonize
         thread.start() # Start the thread
         return thread
@@ -564,31 +573,48 @@ Categories=Utility;
     def bluetooth_ping(self, here_callback, away_callback):
         ''' Ping selected bluetooth device. Perform actions based on RSSI. '''
 
+        hcibin = 'hcitool'
+        getrssicmd = [hcibin, "rssi"]
+
         lost_pings = 0
 
         while not self.ping_stop:
             rssi = -99
-            addr = self.bt_address
 
-            if len(addr) < 17:
+            if len(self.bt_address) < 17:
                 if self.debug:
-                    syslog.syslog("Invalid address %s, not scanning." % addr)
+                    syslog.syslog("Invalid address %s, not scanning." % self.bt_address)
                 self.levelSignal.set_value(0)
                 time.sleep(self.interval)
                 continue
 
-            try:
-                b = BluetoothRSSI(addr=addr)
-                rssi = b.get_rssi()
-                if rssi is None:
-                    rssi = -99
-                self.levelSignal.set_value(10+rssi)
-            except Exception as err:
-                syslog.syslog("Error: %s" % err)
+            if self.debug:
+                syslog.syslog("Running BT scanning command <%s" % ' '.join(getrssicmd + [self.bt_address]))
+            with subprocess.Popen(
+                    args=getrssicmd + [self.bt_address],
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
+                    universal_newlines=True
+            ) as proc:
+
+                for line in iter(proc.stdout.readline, b''):
+                    if self.debug and len(line) > 0:
+                        syslog.syslog("hcitool line: <%s>" % line.strip())
+                    if line.strip() == '': # iter calls until output is ''
+                        break
+                    rssi = int(line.strip().split()[-1])
+
+            if rssi is None:
+                rssi = -99
+            self.levelSignal.set_value(10+rssi)
+            # except Exception as err:
+            #     syslog.syslog("Error: %s" % err)
 
             if rssi is None or rssi < self.threshold:
                 if self.debug:
-                    syslog.syslog("Lost")
+                    syslog.syslog("No connection")
+                    if lost_pings > 0:
+                        syslog.syslog("Lost %s" % lost_pings)
 
                 if self.enabled:
                     lost_pings += 1
@@ -597,9 +623,9 @@ Categories=Utility;
             elif lost_pings > 0:
                 lost_pings = 0
                 here_callback()
-            else:
-                if self.debug:
-                    syslog.syslog("Ping, rssi %s" % rssi)
+            # else:
+            #     if self.debug:
+            #         syslog.syslog("Ping, rssi %s" % rssi)
 
             time.sleep(self.interval)
 
@@ -651,27 +677,29 @@ Categories=Utility;
 def run_user_command(cmd=''):
     ''' Run user supplied command '''
     syslog.syslog("Running user command <%s>." % cmd)
-    subprocess.run(cmd, shell=True)
+    subprocess.run(cmd, shell=True, check=True)
 
 def mute():
     ''' Mute sound '''
     syslog.syslog("Muting sound.")
-    subprocess.run("amixer set Master mute > /dev/null", shell=True)
+    subprocess.run("amixer set Master mute > /dev/null", shell=True, check=True)
 
 def unmute():
     ''' Unmute sound '''
     syslog.syslog("Unmuting sound.")
-    subprocess.run("amixer set Master unmute > /dev/null; amixer set Speaker unmute > /dev/null; amixer set Headphone unmute > /dev/null;", shell=True)
+    subprocess.run("amixer set Master unmute > /dev/null; amixer set Speaker " +\
+        "unmute > /dev/null; amixer set Headphone unmute > /dev/null;",
+        shell=True, check=True)
 
 def pause_music():
     ''' Pause music '''
     syslog.syslog("Pausing music.")
-    subprocess.run("playerctl pause 2> /dev/null", shell=True)
+    subprocess.run("playerctl pause 2> /dev/null", shell=True, check=True)
 
 def resume_music():
     ''' Resume music '''
     syslog.syslog("Resume music.")
-    subprocess.run("playerctl play 2> /dev/null", shell=True)
+    subprocess.run("playerctl play 2> /dev/null", shell=True, check=True)
 
 def unlock():
     ''' Unlock desktop session '''
@@ -684,7 +712,7 @@ def unlock():
     # Hard unlock.
     cmd += "loginctl unlock-session $( loginctl list-sessions --no-legend| cut -f1 -d' ' ); "
 
-    subprocess.run(cmd, shell=True)
+    subprocess.run(cmd, shell=True, check=True)
 
 def lock():
     ''' Lock desktop session '''
@@ -699,7 +727,8 @@ def lock():
         "gsettings set org.gnome.desktop.session idle-delay %s; " % idledelay +\
         "gsettings set org.gnome.desktop.screensaver lock-delay 0; "
 
-    subprocess.run(cmd, shell=True)
+    subprocess.run(cmd, shell=True, check=True)
+
 
 if __name__=="__main__":
     app = BlueDo()
